import com.apama.EngineException;
import com.apama.event.Event;
import com.apama.services.event.EventServiceFactory;
import com.apama.services.event.IEventService;
import com.apama.util.Logger;

/*
 * EventServiceSample.java
 *
 * This sample demonstrates the way to create a simple EventService object and
 * use it to send an event to the Correlator. 
 * 
 * Requirement:
 *
 * The SimpleEvent.mon must be injected to the correlator before running this 
 * example.  The SimpleEvent.mon simply log the message to the Correlator
 * standard output.  You'll see a message similar to this in the Correlator
 * console when you run this example:
 * 
 * 2005-07-27 15:43:10.360 CRIT SimpleEventMon [0] *** SimpleEvent("hi there") received ***
 * 
 *
 * $Copyright(c) 2005 - 2006, 2009-2010, 2012 Progress Software Corporation (PSC). All rights reserved.$
 * $Copyright (c) 2013, 2015 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG
 *
 */

public class EventServiceSample {
	private static final Logger logger = Logger.getLogger(EventServiceSample.class);
	
	/**
	 * Sends the SimpleEvent("hi there") event to the correlator.
	 */
	public void runSample(String host, int port) {
		try
		(
			// get a IEventService instance from the EventServiceFactory
			// IEventService.close() will automatically get called upon exiting the try-with-resources block
			IEventService eventService = EventServiceFactory.createEventService(host, port, "EventServiceSample");
		) {
			// create the simpleEvent object
			Event simpleEvent = new Event("SimpleEvent(\"hi there\")");
			
			// send it
			eventService.sendEvent(simpleEvent);
			
			System.out.println("SimpleEvent(hi there) sent...");  
			System.out.print("\t Please check Correlator console for the string ");
			System.out.println("\t *** SimpleEvent(\"hi there\") received ***");
		} catch (EngineException e) {
			logger.error("Event sending failed: ", e);
		}
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if ((args.length != 2) || (Integer.parseInt(args[1]) <= 0)) {
			// Bad command line given
			System.out.println("Usage: java EventServiceSample <host> <port>");
			return;
		}
		String host = args[0];
		int port = Integer.parseInt(args[1]);
		new EventServiceSample().runSample(host, port);
	}

} 
