/*
 * Simple.java
 *
 * This sample java monitor is used to filter out all stock tick events, for all
 * stock tick symbols, whose traded price is less that 10.0. The implementation 
 * of the Monitor onLoad method creates a new EventExpression to trigger on all
 * tick events whose price is greater than 10.0. The class instantiation is added 
 * as a MatchListener to the EventExpression, emitting the captured stock tick event
 * onto the default channel.
 *
 * $Copyright(c) 2002, 2004-2006 Progress Software Corporation. All Rights Reserved.$ 
 * $Copyright (c) 2013,2015,2019 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$ 
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG 
 *
 * $Revision: 359265 $
 *
 */

import com.apama.jmon.*;
import com.apama.jmon.annotation.*;

@Application(name = "Simple",
			 author = "My Name",
			 version = "1.0",
			 company = "My Company",
			 description = "Simple java monitor sample", 
			 classpath = "")

@MonitorType(description = "A simple java monitor, used to show functionality of a new installation.")
public class Simple implements Monitor, MatchListener {
	private static final long serialVersionUID = 14L;
	/**
	 * No argument constructor used by the jmon framework on 
	 * application loading
	 */
	public Simple() {}
	

	/**
	 * Implementation of the Monitor interface onLoad method. Sets up 
	 * a single event expression looking for all stock trade events 
	 * with a trade price of greater than 10.0. This class instantiation
	 * is added as a match listener to the event expression. 
	 */
	public void onLoad() {			
		EventExpression eventExpr = new EventExpression("all Tick(*, >10.0):t");
		eventExpr.addMatchListener(this);
	}


	/**
	 * Implementation of the MatchListener interface match method. Extracts the
	 * tick event that caused the event expression to trigger and emits the 
	 * event onto the default channel
	 */
	public void match(MatchEvent event) {
		Tick tick = (Tick)event.getMatchingEvents().get("t");
		tick.emit();
	}

}

