/*
 * Tick.java
 *
 * Class to abstract an Apama stock tick event. A stock tick event 
 * describes the trading of a stock, as described by the symbol
 * of the stock being traded, and the price at which the stock was 
 * traded
 *
 * $Copyright(c) 2002, 2004-2006 Progress Software Corporation. All Rights Reserved.$ 
 * $Copyright (c) 2013,2019 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$ 
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG 
 *
 * $Revision: 359265 $
 */

import com.apama.jmon.Event;
import com.apama.jmon.annotation.*;

@EventType(description = "Event which signals a stock trade")
public class Tick extends Event {
	private static final long serialVersionUID = 13L;
	
	/** The stock tick symbol */
	public String name;

	/** The traded price of the stock tick */
	public double price;


	/**
	 * No argument constructor
	 */
	public Tick() {
		this("", 0);
	}


	/**
	 * Construct a tick object and set the name and price
	 * instance variables
	 *
	 * @param name  The stock symbol of the traded stock
	 * @param price The price at which the stock was traded
	 */
	public Tick(String name, double price){
		this.name = name;
		this.price = price;
	}

}

