__pysys_title__   = r""" Java monitor (JMON) injection and execution """
#                        ================================================================================

__pysys_purpose__ = r""" Demonstrates starting an Event Correlator, attaching a receiver to the Correlator, injecting 
	a pre-compiled JMON application, and sending in a series of input events via event file. 
	
	Verification of the test is performed by a file comparison with a reference file for the events received and written 
	to file by the attached receiver.
	"""

__pysys_authors__ = "sample"

# Copyright (c) 2015-2016, 2018, 2020-2021 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors. 
# Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG 

import pysys
from pysys.constants import *

class PySysTest(pysys.basetest.BaseTest):

	def execute(self):
		self.apama.antBuild(self.project.appHome+'/java_monitor/stockwatch/build.xml')
		
		# start the correlator and attach a receiver
		correlator = self.apama.startCorrelator('correlator', java=True)
		correlator.receive(filename='receive.evt')
		
		# inject the stockwatch jmon application into the correlator - note that
		# jar needs to have been built already
		correlator.injectJava(self.output+'/build/stockwatch-jmon.jar')
		
		# send the test events into the correlator
		correlator.send(filenames=[self.project.appHome+'/java_monitor/stockwatch/stockwatch-feed.evt'])
		
		# wait until all the events have been processed and acknowledged by the
		# the receiver
		correlator.flush()

	def validate(self):
		# check the received events against the reference; note that comparing .evt files like this is not recommended 
		# unless your event definitions are unlikely to change, and do not contain any fields (e.g. timestamps) that 
		# do not need to be included in the verification
		self.assertDiff('receive.evt', 'ref_receive.evt')
		
